import {Injectable} from "@angular/core";
import {HttpClient} from "@angular/common/http";
import {BehaviorSubject, from, Observable, Subscriber} from "rxjs";
import {Indicator, Section, Stakeholder, StakeholderInfo, Visibility} from "../entities/stakeholder";
import {HelperFunctions} from "../../utils/HelperFunctions.class";
import {map} from "rxjs/operators";
import {ActivatedRoute} from "@angular/router";
import {properties} from "../../../../environments/environment";
import {CustomOptions} from "../../services/servicesUtils/customOptions.class";

let maps: string[] = ['parameters', 'filters'];

export interface Reorder {
  action: 'moved' | 'added' | 'removed',
  target: string,
  ids: string[];
}

@Injectable({
  providedIn: "root"
})
export class StakeholderService {
  
  private stakeholderSubject: BehaviorSubject<Stakeholder> = null;
  private promise: Promise<any>;
  private sub;
  
  constructor(private http: HttpClient, private route: ActivatedRoute) {
    this.stakeholderSubject = new BehaviorSubject<Stakeholder>(null);
  }
  
  ngOnDestroy() {
    this.clearSubscriptions();
  }
  
  clearSubscriptions() {
    if (this.sub instanceof Subscriber) {
      this.sub.unsubscribe();
    }
  }
  
  getStakeholder(alias: string): Observable<Stakeholder> {
    if (!this.stakeholderSubject.value || this.stakeholderSubject.value.alias !== alias) {
      this.promise = new Promise<any>((resolve, reject) => {
        this.sub = this.http.get<Stakeholder>(properties.monitorServiceAPIURL + '/stakeholder/' + encodeURIComponent(alias), CustomOptions.registryOptions()).pipe(map(stakeholder => {
          return this.formalize(stakeholder);
        })).subscribe(stakeholder => {
          this.stakeholderSubject.next(stakeholder);
          resolve();
        }, error => {
          this.stakeholderSubject.next(null);
          reject();
        });
      });
    }
    return from(this.getStakeholderAsync());
  }
  
  async getStakeholderAsync() {
    await this.promise;
    this.clearSubscriptions();
    return this.stakeholderSubject.getValue();
  }
  
  getAlias(url: string): Observable<string[]> {
    return this.http.get<Stakeholder[]>(url + '/stakeholder/alias', CustomOptions.registryOptions()).pipe(map(stakeholders => {
      return this.formalize(stakeholders);
    }));
  }
  
  getStakeholders(url: string, type: string = null): Observable<(Stakeholder & StakeholderInfo)[]> {
    return this.http.get<Stakeholder[]>(url + '/stakeholder' + ((type) ? ('?type=' + type) : ''), CustomOptions.registryOptions()).pipe(map(stakeholders => {
      return this.formalize(stakeholders);
    }));
  }
  
  getMyStakeholders(url: string, type: string = null): Observable<Stakeholder[]> {
    return this.http.get<Stakeholder[]>(url + '/my-stakeholder' + ((type) ? ('?type=' + type) : ''), CustomOptions.registryOptions()).pipe(map(stakeholders => {
      return this.formalize(stakeholders);
    }));
  }
  
  getDefaultStakeholders(url: string, type: string = null): Observable<Stakeholder[]> {
    return this.http.get<Stakeholder[]>(url + '/stakeholder/default' + ((type) ? ('?type=' + type) : ''), CustomOptions.registryOptions()).pipe(map(stakeholders => {
      return this.formalize(stakeholders);
    }));
  }
  
  buildStakeholder(url: string, stakeholder: Stakeholder): Observable<Stakeholder> {
    return this.http.post<Stakeholder>(url + '/build-stakeholder', stakeholder, CustomOptions.registryOptions()).pipe(map(stakeholder => {
      return this.formalize(stakeholder);
    }));
  }
  
  changeVisibility(url: string, path: string[], visibility: Visibility): Observable<any> {
    return this.http.post<Visibility>(url + '/' + path.join('/') + '/change-visibility' + '?visibility=' + visibility, null, CustomOptions.registryOptions());
  }
  
  saveElement(url: string, element: any, path: string[] = []): Observable<any> {
    path = HelperFunctions.encodeArray(path);
    return this.http.post<any>(url + ((path.length > 0) ? '/' : '') + path.join('/') +
      '/save', element, CustomOptions.registryOptions()).pipe(map(element => {
      return this.formalize(element);
    }));
  }
  
  saveSection(url: string, element: any, path: string[] = [], index: number = -1): Observable<Section> {
    path = HelperFunctions.encodeArray(path);
    return this.http.post<Section>(url + ((path.length > 0) ? '/' : '') + path.join('/') +
      '/save/' + index, element, CustomOptions.registryOptions()).pipe(map(element => {
      return this.formalize(element);
    }));
  }
  
  deleteElement(url: string, path: string[], childrenAction: string = null): Observable<any> {
    path = HelperFunctions.encodeArray(path);
    let params: string = "";
    if (childrenAction) {
      params = "?children=" + childrenAction;
    }
    return this.http.delete<any>(url + '/' + path.join('/') + '/delete' + params, CustomOptions.registryOptions());
  }
  
  reorderIndicators(url: string, path: string[], reorder: Reorder, type: string = 'chart'): Observable<Indicator[]> {
    path = HelperFunctions.encodeArray(path);
    return this.http.post<Indicator[]>(url + '/' + path.join('/') + '/' + type + '/reorder', reorder, CustomOptions.registryOptions()).pipe(map(indicators => {
      return this.formalize(indicators);
    }));
  }
  
  getStakeholderAsObservable(): Observable<Stakeholder> {
    return this.stakeholderSubject.asObservable();
  }
  
  setStakeholder(stakeholder: Stakeholder) {
    this.stakeholderSubject.next(stakeholder);
  }
  
  private formalize(element: any) {
    return HelperFunctions.copy(element);
  }
}
